<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

//
// NOTE:
//
// This instance of VOMSCompatibility implementation is based on PHP-SOAP
// in  contrast to manual XML creation
//
// Unfortunately, PHP SOAP is not compatible with AXIS 1.2 used in Java VOMS-Admin,
// only with 1.3 :-(
//
// Several problems arose while working on implementation: 
//   - WSDL contains overloaded methods which is no longer supported since WSDL 1.2
//   - PHP SOAP returns 'SOAP-ENV' in contrast to 'soapenv' prefix
//   - Response structure left the same, but node names differs
//
// Specification said that prefixes aren't required at all and node names are not 
// valuable in WSDL-mode. So if client implementation uses SOAP call - result will 
// be handled properly. But I'm not sure, that gLite implementations does not 
// use own XML parsers (especially for GET method) hardcoded to node names and so on.
// ( at least edg-mkgridmap uses hardcoded tag names to parse VOMS output )
//
// Looking at all this confusing things I have decided to stick on 'manual XML creation'
// that will produce output identical to Java VOMS-Admin as primary VOMSCompatibility 
// medthod, but include VOMSCompatibility2 into distribution to get available for 
// proper SOAP clients. WSDL has been pathched to eliminate overloaded methods.
//

	error_reporting(E_ALL); ini_set("display_errors", 1);
	ini_set("soap.wsdl_cache_enabled", "1");
	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	require("VOMSSOAP.php");

	// WSDL files
	$wsdl_file = 'wsdl/voms-compatibility-2.0.2.wsdl';
	$wsdl_server_file = 'wsdl/voms-compatibility-2.0.2-server.wsdl';

	// WSDL URL
	$wsdl_url = 'http://'.$_SERVER['HTTP_HOST'].preg_replace('/\?.*/','',$_SERVER['REQUEST_URI']);

	// return WSDL
	if ( isset($_GET['wsdl']) ) {
		$wsdl = file_get_contents($wsdl_file);
		echo preg_replace('/@@ENDPOINT@@/',$wsdl_url,$wsdl);
		exit(0);
	}

	// define handler class
	class VOMSCompatibility extends VOMSSOAP {
		public function getGridmapUsers($container = NULL) {
			if ( ! $container ) { 
				// check permissions
				$this->acl_check(get_ACL_permissions(), "membership", "r" );
				return getVOMembers();
			}
			// parse container
			$pcont = $this->parse_container($container);
			$group_name = '/' . implode('/',$pcont['groups']);
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "membership", "r" );
			// get role ID
			if ( $pcont['role'] !== NULL ) {
				$rid = $this->get_rid($pcont['role']);
			} else $rid = NULL;
			// get container users
			$cusers = getAllUsersInfo($gid, $rid);
			// form result
			$result = array ();
			foreach ( $cusers as $userinfo ) $result[] = $userinfo['DN'];
			return $result;
		}
	}

	// create SAOP server object
	$server = new SoapServer($wsdl_server_file, array(
			'soap_version' => SOAP_1_1 )); 
	$server->setClass('VOMSCompatibility',$wsdl_file);

	// handle client request
	$data = VOMSSOAP::getSOAPRequestData();
	if ( ! $data ) $server->fault('ns1:Client',"Error: method not specified");
	
	// overcome to handle calls with and without parameters for previously overloaded methods
	$regex_empty_req = '/<([^>\/ ]*)(getGridmapUsers)([^>\/]*)(\/>|><\/([^>]+)>)/';
	$subst_empty_req = '<\1\2\3><in0></in0></\1\2>';
	$data = preg_replace($regex_empty_req, $subst_empty_req, $data);

	// catching SOAPFault exceptions does not work for some reson
	// here the hack to return proper exceptions output
	ob_start();
	$server->handle($data);
	$soapResponse = ob_get_contents();
	ob_end_clean();

	header('HTTP/1.0 200 OK');
	echo $soapResponse;
?>
