<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	define('NO_GET_PERMISSIONS',1);
	define('SOAP_UPDATOR_ID','1:SOAP');

	require_once("acl.php");
	require("config.inc");
	require("get_vo_configuration.php");
	require("https_client_auth.php");

	// define base handler class
	class VOMSSOAP {
		protected $major_version = 0;
		protected $minor_version = 0;
		protected $patch_version = 0;
		protected $vo;

		function __construct ($wsdl_file, $vo = NULL) {
			$matches = array();
			if ( preg_match('/[^0-9]*([0-9])\.([0-9])\.([0-9])\.wsdl/',$wsdl_file, $matches) ) {
				$this->major_version = $matches[1];
				$this->minor_version = $matches[2];
				$this->patch_version = $matches[3];
				$this->vo = ($vo === NULL ) ? $GLOBALS['vo'] : $vo;
			}
		}

		// define friendly SOAPFault interface
		protected function request_failed ($e_name, $e_descr) {
			throw new SoapFault($e_name, $e_descr, ob_get_contents(), "");
		}

		// define friendly SOAPFault permissions checking interface
		protected function acl_check ($perm, $op, $rw) {
			if ( ! isACLallow($perm, $op, $rw) ) {
				$this->request_failed("Server.authException",
					sprintf("AccessDeniedException: operation is not allowed due to %s:%s ACL",$op,$rw));
			}
		}
		
		// check name for not allowed characters
		protected function name_check ($argname) {
			if ( ! preg_match('/^[a-zA-Z0-9\-\_\.]+$/', $argname) ) {
				$this->request_failed("Server.userException",
					sprintf("VOMSSyntaxException: Syntax error in name passed: %s", $argname));
			}
		}

		// check group name contains /vo part and return array of group hierarchy
		protected function parse_group_name ($group_name, $nosyntax = false ) {
			$gname_arr = explode('/',$group_name);
			if ( ! $nosyntax ) if ( $gname_arr[1] !== $this->vo ) {
				$this->request_failed("Server.userException",
					sprintf("VOMSSyntaxException: Syntax error in group name: %s",$group_name));
			}
			return $gname_arr;
		}

		protected function parse_container ($container) {
			$cont_arr = explode('/',$container);
			$capability = NULL;
			$role = NULL;
			$groups = array ();
			// get capability if any
			$contp = array_pop($cont_arr);
			if ( $contp ) {
				if ( preg_match('/^C(apability|APABILITY)=/',$contp) ) {
					$capability = preg_replace('/^C(apability|APABILITY)=/','',$contp);
					if ( $capability == 'NULL' ) $capability = NULL;
					$contp = array_pop($cont_arr); 
				}
			} else return NULL;
			// get role if any
			if ( $contp ) { 
				if ( preg_match('/^R(ole|OLE)=/',$contp) ) {
					$role = preg_replace('/^R(ole|OLE)=/','',$contp);
					if ( $role == 'NULL' ) $role = NULL;
					$contp = array_pop($cont_arr);
				} 
			} else return NULL;
			// get all groups array
			while ( $contp ) {
				$ccontp = preg_replace(array('/^VO=/','/^G(roup|ROUP)=/'),array('',''),$contp);
				array_unshift($groups,$ccontp);
				$contp = array_pop($cont_arr);
			}
			return  array (	"groups" => $groups,
					"role" => $role,
					"capability" => $capability );
		}

		// get group ID by name
		protected function get_gid ($group_name) {
			$gid = getGroupByName($group_name);
			if ( ! $gid ) $this->request_failed("Server.userException",
				sprintf("NoSuchGroupException: Group '%s' is not defined for this vo.",$group_name));	
			return $gid;
		}

		// get role ID by name
		protected function get_rid ($role_name) {
			$rname = preg_replace(array('/Role=/','/ROLE=/'), array('',''), $role_name);
			$rid = getRoleId($rname);
			if ( ! $rid ) $this->request_failed("Server.userException",
				sprintf("NoSuchRoleException: Role '%s' is not defined for this vo.",$rname));
			return $rid;
		}

		// get user ID by DN/CA
		protected function get_uid ($dn, $ca) {
			$uid = getUserID($dn, $ca);
			if ( ! $uid ) $this->request_failed ("Server.userException",
				sprintf("NoSuchUserException: User %s,%s is not found in VO database", $dn, $ca));
			return $uid;
		}

		// get attribute ID by name
		protected function get_aid ($attr_name) {
			$aid = getAttributeId ($attr_name);
			if ( ! $aid ) $this->request_failed ("Server.userException",
				sprintf("NoSuchAttributeException: Attribute %s is not found in VO database", $attr_name));
			return $aid;
		}

		// unsuported methods exception
		public function __call($method, $args) {
			$this->request_failed ("Server.handlerException", 
				sprintf("NoSuchMethodException: Requested method %s(%s) is not supported", 
					$method, implode(', ', $args)));
		}

		// WSDL version numbers
		public function getMajorVersionNumber() {
			return $this->major_version;
		}

		public function getMinorVersionNumber() {
			return $this->minor_version;
		}

		public function getPatchVersionNumber() {
			return $this->patch_version;
		}

		// handle overloaded parameters
		protected function getSOAPOverloadedParameter($in0, $names_chain, $method) {
			// handle complex class vs soapenc:string
			if ( is_object($in0) ) {
				// modified non-overloaded WSDL expect object only
				$param1 = is_array($names_chain) ? $names_chain[0] : $names_chain;
				if ( ! isset($in0->$param1) ) {
					// Non-object records without type specified (e.g. <name>class</name>)
					// treated as empty object. In case of not-WSDL clients like original
					// voms-admin-client python tool this is exactly what we have :-(
					// Here is the dirty manual XML parsing hack to get value.
					global $data;
					$matches = array();
					$regex = '@>([^<]+)</[^>]+></([^>:]+:)?'.$method.'>@';
					preg_match($regex,$data,$matches);
					if ( count($matches) ){
						$result = $matches[1];
					} else {
						$this->request_failed ("Server.serverException",
							"ParseException: Failed to process SOAP request.");
					}
				} else {
					// Object passed as expected
					$result = $in0;
					if ( is_array($names_chain) ) {
						foreach ( $names_chain as $param ) {
							$result = $result->$param;
						}
					} else {
						$result = $in0->$names_chain;
					}
				}
			} else {
				// If record type specified (e.g. <name type="xsd:string">class</name>)
				// we can use string directly even if WSDL expect object.
				$result = $in0;
			}
			return $result;
		}

		// get SAOP client request XML data
		public static function getSOAPRequestData() {
			if ($_SERVER["REQUEST_METHOD"] == "POST") {
				// POST SOAP
				$data = file_get_contents('php://input');
			} else {
				// GET request
				// obtain method name
				if ( isset($_GET['method']) ) {
					$method = $_GET['method'];
					$param = next($_GET);
					while (( $param == $GLOBALS['vo'] ) || ( $param == $method ))
					 	$param = next($_GET);
				} else return 0;
				// Construct parameters XML for request
				$req = sprintf('<ns1:%s>',$method);
				$pcnt = 0;
				while ( $param ) {
					$req .= sprintf('<in%d xsi:type="xsd:string">%s</in%d>',$pcnt,$param,$pcnt);
					$pcnt++;
					$param = next($_GET);
				}
				$req .= sprintf('</ns1:%s>',$method);
				// Create SOAP XML 
				// ?'.'> is for vim proper highlighting :-)
				$data = sprintf('<?xml version="1.0" encoding="UTF-8"?'.'>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://glite.org/wsdl/services/org.glite.security.voms.service.admin" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"><SOAP-ENV:Body>%s</SOAP-ENV:Body></SOAP-ENV:Envelope>', $req);
			}
			return $data;
		} 
	}

?>
