<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//
	////////////////////////////////////////////////////////////////
	// ACL Management (required before functions import for SQL operation with ACL)
	////////////////////////////////////////////////////////////////

	/* Decode permissions intenger to hash
	   return (hash) of permissions flags */
	function decodeACLPermissions ( $permissions ) {
		$pstring = sprintf("%021b", $permissions);
		$parr["container"]["r"] = $pstring[20];
		$parr["container"]["w"] = $pstring[19];
		$parr["membership"]["r"] = $pstring[18];
		$parr["membership"]["w"] = $pstring[17];
		$parr["acl"]["r"] = $pstring[16];
		$parr["acl"]["w"] = $pstring[15];
		$parr["acl"]["d"] = $pstring[14];
		$parr["requests"]["r"] = $pstring[13];
		$parr["requests"]["w"] = $pstring[12];
		$parr["attributes"]["r"] = $pstring[11];
		$parr["attributes"]["w"] = $pstring[10];
		$parr["preferences"]["r"] = $pstring[9];
		$parr["preferences"]["w"] = $pstring[8];
		return $parr;
	}

	/* Encode permissions hash to database integer 
	   return (int) permissions db value */
	function constructACLPermissions( $perm_arr ) {
		$perm = 0;
		if ( isset( $perm_arr["containerr"] ) ) $perm += 1;
		if ( isset( $perm_arr["containerw"] ) ) $perm += 2;
		if ( isset( $perm_arr["membershipr"] ) ) $perm += 4;
		if ( isset( $perm_arr["membershipw"] ) ) $perm += 8;
		if ( isset( $perm_arr["aclr"] ) ) $perm += 16;
		if ( isset( $perm_arr["aclw"] ) ) $perm += 32;
		if ( isset( $perm_arr["acld"] ) ) $perm += 64;
		if ( isset( $perm_arr["requestsr"] ) ) $perm += 128;
		if ( isset( $perm_arr["requestsw"] ) ) $perm += 256;
		if ( isset( $perm_arr["attributesr"] ) ) $perm += 512;
		if ( isset( $perm_arr["attributesw"] ) ) $perm += 1024;
		if ( isset( $perm_arr["preferencesr"] ) ) $perm += 2048;
		if ( isset( $perm_arr["preferencesw"] ) ) $perm += 4096;
		return $perm;
	}

	/* Check database version is new anough to operate
	   return (bool) required */
	function checkDBSchema () {
		global $pva_dbschema_version;
		if ( ! isset($pva_dbschema_version) ) $pva_dbschema_version = 0;
		$REQUIRED_DBSCHEMA_VERSION = 65;
		return ( $pva_dbschema_version >= $REQUIRED_DBSCHEMA_VERSION );
	}

	/* Check ACL rights allow operation
	   return (bool) permission check result */
	function isACLallow ( $permissions_a, $class, $op, $quiet = 0 ) {
		global $voms_insufficient_read;
		// prohibit write on database schema mismatch
		if (( ! checkDBSchema() ) && ( $op === "w" )) return 0;
		// use 'W' to check write permissions ignoring detabase schema
		if ( $op === "W" ) $op = "w";
		$result = ( isset($permissions_a[$class][$op]) ) ? $permissions_a[$class][$op] : 0;
		if ( ( $op === "r" ) && ( $result == 0 ) && ( $quiet == 0 )) printf($voms_insufficient_read, $class);
		return $result;
	}

	/* Get group permissions for specified user
	   return (array) [permissions][rw] = [isset] */
	function get_ACL_permissions ($gid = 1, $dn = NULL, $ca = NULL) {
		if ( $dn === NULL ) $dn = $GLOBALS['USER_DN'];
		if ( $ca === NULL ) $ca = $GLOBALS['USER_CA'];
		$member = checkMember($dn, $ca);
		$perm = getProperUserACL($dn, $ca, $member, $gid);
		return decodeACLPermissions($perm);
	}


?>
